@@ -6,15 +6,16 @@ import (
 	"sync"
 	"time"
 
-	consolev1 "github.com/openshift/api/console/v1"
-	conditionsv1 "github.com/openshift/custom-resource-status/conditions/v1"
+	log "github.com/go-logr/logr"
 	corev1 "k8s.io/api/core/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
-	kubevirtv1 "kubevirt.io/client-go/api/v1"
-	cdiv1beta1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	logf "sigs.k8s.io/controller-runtime/pkg/log"
 
+	kubevirtcorev1 "kubevirt.io/api/core/v1"
+	cdiv1beta1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
+
 	hcov1beta1 "github.com/kubevirt/hyperconverged-cluster-operator/pkg/apis/hco/v1beta1"
 	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/controller/common"
 	"github.com/kubevirt/hyperconverged-cluster-operator/pkg/metrics"
@@ -40,8 +41,8 @@ type OperandHandler struct {
 	client   client.Client
 	operands []Operand
 	// save for deletions
-	quickStartObjects []*consolev1.ConsoleQuickStart
-	eventEmitter      hcoutil.EventEmitter
+	objects      []client.Object
+	eventEmitter hcoutil.EventEmitter
 }
 
 func NewOperandHandler(client client.Client, scheme *runtime.Scheme, isOpenshiftCluster bool, eventEmitter hcoutil.EventEmitter) *OperandHandler {
@@ -51,9 +52,6 @@ func NewOperandHandler(client client.Client, scheme *runtime.Scheme, isOpenshift
 		(*genericOperand)(newCdiHandler(client, scheme)),
 		(*genericOperand)(newStorageConfigHandler(client, scheme)),
 		(*genericOperand)(newCnaHandler(client, scheme)),
-		(*genericOperand)(newVmImportHandler(client, scheme)),
-		(*genericOperand)(newImsConfigHandler(client, scheme)),
-		newKubeVirtCmHandler(client, eventEmitter),
 	}
 
 	if isOpenshiftCluster {
@@ -62,7 +60,9 @@ func NewOperandHandler(client client.Client, scheme *runtime.Scheme, isOpenshift
 			(*genericOperand)(newMetricsServiceHandler(client, scheme)),
 			(*genericOperand)(newMetricsServiceMonitorHandler(client, scheme)),
 			(*genericOperand)(newMonitoringPrometheusRuleHandler(client, scheme)),
-			(*genericOperand)(newCLIDownloadHandler(client, scheme)),
+			(*genericOperand)(newCliDownloadHandler(client, scheme)),
+			(*genericOperand)(newCliDownloadsRouteHandler(client, scheme)),
+			(*genericOperand)(newCliDownloadsServiceHandler(client, scheme)),
 		}...)
 	}
 
@@ -76,25 +76,54 @@ func NewOperandHandler(client client.Client, scheme *runtime.Scheme, isOpenshift
 // The k8s client is not available when calling to NewOperandHandler.
 // Initial operations that need to read/write from the cluster can only be done when the client is already working.
 func (h *OperandHandler) FirstUseInitiation(scheme *runtime.Scheme, isOpenshiftCluster bool, hc *hcov1beta1.HyperConverged) {
+	h.objects = make([]client.Object, 0)
 	if isOpenshiftCluster {
-		qsHandlers, err := getQuickStartHandlers(logger, h.client, scheme, hc)
-		if numQs := len(qsHandlers); numQs > 0 {
-			h.quickStartObjects = make([]*consolev1.ConsoleQuickStart, numQs)
-			for i, op := range qsHandlers {
-				qs, err := op.(*genericOperand).hooks.getFullCr(hc)
-				if err != nil {
-					logger.Error(err, "can't create ConsoleQuickStarts object")
-					continue
-				}
+		h.addOperands(scheme, hc, getQuickStartHandlers)
+		h.addOperands(scheme, hc, getDashboardHandlers)
+		h.addOperands(scheme, hc, getImageStreamHandlers)
+		h.addOperands(scheme, hc, newVirtioWinCmHandler)
+		h.addOperands(scheme, hc, newVirtioWinCmReaderRoleHandler)
+		h.addOperands(scheme, hc, newVirtioWinCmReaderRoleBindingHandler)
+	}
+	// Role and RoleBinding for kvStorage Config Map should be created both on Openshift and plain k8s
+	h.addOperands(scheme, hc, NewConfigReaderRoleHandler)
+	h.addOperands(scheme, hc, newConfigReaderRoleBindingHandler)
 
-				h.quickStartObjects[i] = qs.(*consolev1.ConsoleQuickStart)
+}
+
+func (h *OperandHandler) GetQuickStartNames() []string {
+	return quickstartNames
+}
+
+type GetHandler func(logger log.Logger, Client client.Client, Scheme *runtime.Scheme, hc *hcov1beta1.HyperConverged) ([]Operand, error)
+
+func (h *OperandHandler) addOperands(scheme *runtime.Scheme, hc *hcov1beta1.HyperConverged, getHandler GetHandler) {
+	handlers, err := getHandler(logger, h.client, scheme, hc)
+	if err != nil {
+		logger.Error(err, "can't create ConsoleQuickStarts objects")
+	} else if len(handlers) > 0 {
+		for _, handler := range handlers {
+			var (
+				obj client.Object
+				err error
+			)
+
+			switch h := handler.(type) {
+			case *genericOperand:
+				obj, err = h.hooks.getFullCr(hc)
+			case *imageStreamOperand:
+				obj, err = h.operand.hooks.getFullCr(hc)
+			default:
+				err = fmt.Errorf("unknown handler with type %v", h)
 			}
+
+			if err != nil {
+				logger.Error(err, "can't create object")
+				continue
+			}
+			h.objects = append(h.objects, obj)
 		}
-		if err != nil {
-			logger.Error(err, "can't create ConsoleQuickStarts objects")
-		} else if len(qsHandlers) > 0 {
-			h.operands = append(h.operands, qsHandlers...)
-		}
+		h.operands = append(h.operands, handlers...)
 	}
 }
 
@@ -102,12 +131,15 @@ func (h OperandHandler) Ensure(req *common.HcoRequest) error {
 	for _, handler := range h.operands {
 		res := handler.ensure(req)
 		if res.Err != nil {
+			req.Logger.Error(res.Err, "failed to ensure an operand")
+
 			req.ComponentUpgradeInProgress = false
-			req.Conditions.SetStatusCondition(conditionsv1.Condition{
-				Type:    hcov1beta1.ConditionReconcileComplete,
-				Status:  corev1.ConditionFalse,
-				Reason:  reconcileFailed,
-				Message: fmt.Sprintf("Error while reconciling: %v", res.Err),
+			req.Conditions.SetStatusCondition(metav1.Condition{
+				Type:               hcov1beta1.ConditionReconcileComplete,
+				Status:             metav1.ConditionFalse,
+				Reason:             reconcileFailed,
+				Message:            fmt.Sprintf("Error while reconciling: %v", res.Err),
+				ObservedGeneration: req.Instance.Generation,
 			})
 			return res.Err
 		}
@@ -119,8 +151,13 @@ func (h OperandHandler) Ensure(req *common.HcoRequest) error {
 				h.eventEmitter.EmitEvent(req.Instance, corev1.EventTypeNormal, "Updated", fmt.Sprintf("Updated %s %s", res.Type, res.Name))
 			} else {
 				h.eventEmitter.EmitEvent(req.Instance, corev1.EventTypeWarning, "Overwritten", fmt.Sprintf("Overwritten %s %s", res.Type, res.Name))
-				metrics.HcoMetrics.IncOverwrittenModifications(res.Type, res.Name)
+				err := metrics.HcoMetrics.IncOverwrittenModifications(res.Type, res.Name)
+				if err != nil {
+					req.Logger.Error(err, "couldn't update 'OverwrittenModifications' metric")
+				}
 			}
+		} else if res.Deleted {
+			h.eventEmitter.EmitEvent(req.Instance, corev1.EventTypeNormal, "Killing", fmt.Sprintf("Removed %s %s", res.Type, res.Name))
 		}
 
 		req.ComponentUpgradeInProgress = req.ComponentUpgradeInProgress && res.UpgradeDone
@@ -142,14 +179,11 @@ func (h OperandHandler) EnsureDeleted(req *common.HcoRequest) error {
 		NewKubeVirtWithNameOnly(req.Instance),
 		NewCDIWithNameOnly(req.Instance),
 		NewNetworkAddonsWithNameOnly(req.Instance),
-		NewSSP(req.Instance),
+		NewSSPWithNameOnly(req.Instance),
 		NewConsoleCLIDownload(req.Instance),
-		NewVMImportForCR(req.Instance),
 	}
 
-	for _, qs := range h.quickStartObjects {
-		resources = append(resources, qs)
-	}
+	resources = append(resources, h.objects...)
 
 	wg.Add(len(resources))
 
@@ -161,13 +195,13 @@ func (h OperandHandler) EnsureDeleted(req *common.HcoRequest) error {
 	for _, res := range resources {
 		go func(o client.Object, wgr *sync.WaitGroup) {
 			defer wgr.Done()
-			err := hcoutil.EnsureDeleted(tCtx, h.client, o, req.Instance.Name, req.Logger, false, true)
+			deleted, err := hcoutil.EnsureDeleted(tCtx, h.client, o, req.Instance.Name, req.Logger, false, true)
 			if err != nil {
 				req.Logger.Error(err, "Failed to manually delete objects")
 				errT := ErrHCOUninstall
 				errMsg := uninstallHCOErrorMsg
 				switch o.(type) {
-				case *kubevirtv1.KubeVirt:
+				case *kubevirtcorev1.KubeVirt:
 					errT = ErrVirtUninstall
 					errMsg = uninstallVirtErrorMsg + err.Error()
 				case *cdiv1beta1.CDI:
@@ -177,7 +211,7 @@ func (h OperandHandler) EnsureDeleted(req *common.HcoRequest) error {
 
 				h.eventEmitter.EmitEvent(req.Instance, corev1.EventTypeWarning, errT, errMsg)
 				errorCh <- err
-			} else {
+			} else if deleted {
 				key := client.ObjectKeyFromObject(o)
 				h.eventEmitter.EmitEvent(req.Instance, corev1.EventTypeNormal, "Killing", fmt.Sprintf("Removed %s %s", o.GetObjectKind().GroupVersionKind().Kind, key.Name))
 			}
